##
## test_accesscodes.py
## Login : <uli@pu.smp.net>
## Started on  Sun Jun 12 13:07:58 2011 Uli Fouquet
## $Id$
## 
## Copyright (C) 2011 Uli Fouquet
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import shutil
import tempfile
import unittest
from zope.app.testing.functional import FunctionalTestCase
from zope.component.hooks import setSite
from waeup.sirp.app import University
from waeup.sirp.testing import FunctionalLayer
from waeup.sirp.accesscodes.accesscodes import (
    AccessCodeBatch, get_access_code, invalidate_accesscode,
    disable_accesscode)

class AccessCodeHelpersTests(FunctionalTestCase):

    layer = FunctionalLayer

    def setUp(self):
        super(AccessCodeHelpersTests, self).setUp()

        # Prepopulate ZODB
        app = University()
        self.dc_root = tempfile.mkdtemp()
        app['datacenter'].setStoragePath(self.dc_root)

        # Prepopulate the ZODB...
        self.getRootFolder()['app'] = app
        self.app = self.getRootFolder()['app']

        # Create batch
        batch = AccessCodeBatch('now', 'manfred', 'APP', 6.6, 0)
        self.app['accesscodes'].addBatch(batch)

        # Fill batch with accesscodes
        batch.addAccessCode(0, '11111111')
        batch.addAccessCode(1, '22222222')
        batch.addAccessCode(2, '33333333')
        self.ac1 = batch.getAccessCode('APP-1-11111111')
        self.ac2 = batch.getAccessCode('APP-1-22222222')
        self.ac3 = batch.getAccessCode('APP-1-33333333')

        setSite(self.app)
        return

    def tearDown(self):
        shutil.rmtree(self.dc_root)
        super(AccessCodeHelpersTests, self).tearDown()
        return

    def test_get_access_code(self):
        ac = get_access_code('APP-1-11111111')
        assert ac is self.ac1

    def test_get_access_code_not_string(self):
        ac = get_access_code(object())
        assert ac is None

    def test_get_access_code_no_proper_pin(self):
        ac = get_access_code('APP-without_pin')
        assert ac is None

    def test_get_access_code_invalid_batch_num(self):
        ac = get_access_code('APP-invalid-11111111')
        assert ac is None

    def test_get_access_code_invalid_pin(self):
        ac = get_access_code('APP-1-notexistent')
        assert ac is None

    def test_invalidate_accesscode(self):
        assert self.ac1._invalidation_date is None
        invalidate_accesscode('APP-1-11111111')
        assert self.ac1._invalidation_date is not None

    def test_disable_accesscode_unused(self):
        assert self.ac1._disabled is False
        disable_accesscode('APP-1-11111111')
        assert self.ac1._disabled is True

    def test_disable_accesscode_used(self):
        assert self.ac1._disabled is False
        invalidate_accesscode('APP-1-11111111')
        disable_accesscode('APP-1-11111111')
        assert self.ac1._disabled is True
